<?php

namespace Insign\PostCore\Model\Config\Source\Barcode;

use Magento\Framework\View\Element\Context;
use Magento\Framework\View\Element\Html\Select;

class DeliveryMethodsColumn extends Select implements \Magento\Framework\Data\OptionSourceInterface
{
    /**
     * @var \Magento\Framework\App\Config\ScopeConfigInterface
     */
    private $scopeConfig;

    /**
     * DeliveryMethodsColumn constructor.
     * @param \Magento\Framework\App\Config\ScopeConfigInterface $scopeConfig
     * @param Context $context
     * @param array $data
     */
    public function __construct(
        \Magento\Framework\App\Config\ScopeConfigInterface $scopeConfig,
        Context $context,
        array $data = []
    ) {
        parent::__construct($context, $data);
        $this->scopeConfig = $scopeConfig;
    }

    /**
     * @return array
     */
    private function getSourceOptions(): array
    {
        return [
            ['value' => 'ECO', 'label' => __('PostPac Economy')],//
            ['value' => 'PRI', 'label' => __('PostPac Priority')],//
            ['value' => 'SP, ECO', 'label' => __('Bulky goods Economy')],//
            ['value' => 'SP, PRI', 'label' => __('Bulky goods Priority')],//
            ['value' => 'SEM', 'label' => __('Swiss-Express "Moon"')],//
            ['value' => 'GAS, ECO', 'label' => __('PostPac Economy GAS')],//
            ['value' => 'GAS, PRI', 'label' => __('PostPac Priority GAS')],//
        ];
    }

//    /**
//     * Render block HTML
//     *
//     * @return string
//     */
    public function _toHtml(): string
    {
        if (!$this->getOptions()) {
            $this->setOptions($this->getSourceOptions());
        }
        return parent::_toHtml();
    }

    /**
     * @inheritDoc
     */
    public function toOptionArray(): array
    {
        return $this->getSourceOptions();
    }

    /**
     * Set "name" for <select> element
     *
     * @param string $value
     * @return $this
     */
    public function setInputName(string $value): DeliveryMethodsColumn
    {
        return $this->setName($value);
    }

    /**
     * Set "id" for <select> element
     *
     * @param string $value
     * @return $this
     */
    public function setInputId(string $value): DeliveryMethodsColumn
    {
        return $this->setId($value);
    }
}
