<?php

namespace Insign\PostCore\Model;

use Insign\PostCore\Api\GuestAddressValidationInterface;
use Insign\PostCore\Helper\TokenHelper;
use Insign\PostCore\Util\AddressMapper;
use Insign\PostCore\Util\OAuth2\AbstractProvider;
use Insign\PostCore\Util\OAuth2\AccessTokenManager;
use Insign\PostCore\Util\OAuth2\AddressValidationProvider;
use Magento\Checkout\Model\Session;
use Magento\Quote\Api\Data\AddressInterface;

class GuestAddressValidationManagement implements GuestAddressValidationInterface
{
    /**
     * @var int
     */
    private $requestCounter = 0;

    /**
     * @var AccessTokenManager
     */
    protected $accessTokenManager;
    /**
     * @var AddressValidationProvider
     */
    protected $oauth2Provider;
    /**
     * @var AddressMapper
     */
    protected $addressMapper;
    /**
     * @var Session
     */
    private $checkoutSession;
    /**
     * @var TokenHelper
     */
    private $tokenHelper;

    /**
     * GuestAddressValidationManagement constructor.
     * @param AccessTokenManager $accessTokenManager
     * @param TokenHelper $tokenHelper
     * @param AddressValidationProvider $oauth2Provider
     * @param AddressMapper $addressMapper
     * @param Session $checkoutSession
     */
    public function __construct(
        AccessTokenManager $accessTokenManager,
        TokenHelper $tokenHelper,
        AddressValidationProvider $oauth2Provider,
        AddressMapper $addressMapper,
        Session $checkoutSession
    ) {
        $this->accessTokenManager = $accessTokenManager;
        $this->tokenHelper = $tokenHelper;
        $this->oauth2Provider = $oauth2Provider;
        $this->addressMapper = $addressMapper;
        $this->checkoutSession = $checkoutSession;
    }

    /**
     * @inheritdoc
     */
    public function validateByAddress($cartId, AddressInterface $address): string
    {
        if (!$this->tokenHelper->isValidationEnable()) {
            return false;
        }

        $options = [];
        $streetNumber = '';
        $street = '';
        if (preg_match('/^([^\d]*[^\d\s]) *(\d.*)$/', $address->getStreet()[0], $match)) {
            if (is_array($match)) {
                $streetNumber = end($match);
                $arrayStreet = array_slice($match, 1, 1, true);
                $street = reset($arrayStreet);
            }
        }
//        elseif (isset($address->getStreet()[1])) {
//            $arrayStreetHouse = $address->getStreet();
//            $streetNumber = end($arrayStreetHouse);
//            $street = reset($arrayStreetHouse);
//        }

        $customerData = [
            'firstname'=> $address->getFirstname(),
            'lastname'=> $address->getLastname(),
            'street'=> $street,
            'nr'=> $streetNumber,
            'postcode'=> $address->getPostcode(),
            'city'=> $address->getCity()
        ];
        $addressMapper = $this->addressMapper->getAddressData($customerData);
        $url = $this->tokenHelper->getAddressValidationEndpoint();
        $options['json'] = $addressMapper;
        $accessToken = $this->accessTokenManager->getAccessToken($this->tokenHelper->getAddressValidationScope());
        if (!isset($accessToken)) {
            $accessToken = $this->oauth2Provider->createAccessToken();
        }

        try {
            $response = $this->oauth2Provider->createAuthenticatedRequest(AbstractProvider::METHOD_HTTP_POST, $url, $accessToken, $options);
            $data = json_decode((string)$response->getBody(), true);
            if (isset($data['quality'])) {
                $this->checkoutSession->setVarValue($data['quality']);
                return $data['quality'];
//                switch ($data['quality']) {
//                    case "CERTIFIED":
//                        return "CERTIFIED";
//                    case "USABLE":
//                    case "FIXED":
//                        return "VALID";
//                    case "UNUSABLE":
//                    case "COMPROMISED":
//                        return "NOT VALID";
//                    default:
//                        return $data['quality'];
//                }
            }
        } catch (\Exception $e) {
            if ($e->getCode() == 401 && $this->requestCounter < 1) {
                $this->requestCounter++;
                $this->oauth2Provider->createAccessToken();
                return $this->validateByAddress($cartId, $address);
            } else {
                return "ERROR";
            }
        }
        return $response;
    }
}
