<?php

namespace Insign\PostCore\Util\OAuth2;

use Insign\PostCore\Helper\TokenHelper;
use Insign\PostCore\Logger\Log;
use Insign\PostCore\Model\Token;
use Insign\PostCore\Model\TokenFactory;
use Insign\PostCore\Model\TokenRepository;
use Magento\Framework\Api\FilterBuilder;
use Magento\Framework\Api\SearchCriteriaBuilder;
use Magento\Framework\Exception\CouldNotSaveException;
use Magento\Vault\Model\PaymentTokenRepository;

class AccessTokenManager
{
    /** @var TokenFactory  */
    protected $tokenFactory;

    /** @var PaymentTokenRepository */
    protected $tokenRepository;

    /** @var TokenHelper  */
    protected $tokenHelper;

    /** @var SearchCriteriaBuilder  */
    protected $searchCriteriaBuilder;

    /** @var FilterBuilder  */
    protected $filterBuilder;

    /** @var Log */
    protected $logger;

    /**
     * AccessTokenManager constructor.
     * @param TokenFactory $tokenFactory
     * @param TokenRepository $tokenRepository
     * @param TokenHelper $tokenHelper
     * @param SearchCriteriaBuilder $searchCriteriaBuilder
     * @param FilterBuilder $filterBuilder
     * @param Log $logger
     */
    public function __construct(
        TokenFactory $tokenFactory,
        TokenRepository $tokenRepository,
        TokenHelper $tokenHelper,
        SearchCriteriaBuilder $searchCriteriaBuilder,
        FilterBuilder $filterBuilder,
        Log $logger
    ) {
        $this->tokenFactory = $tokenFactory;
        $this->tokenRepository= $tokenRepository;
        $this->tokenHelper = $tokenHelper;
        $this->searchCriteriaBuilder = $searchCriteriaBuilder;
        $this->filterBuilder = $filterBuilder;
        $this->logger = $logger;
    }

    public function getAccessToken($scope)
    {
        $filter = $this->filterBuilder
            ->setField(Token::SCOPE)
            ->setConditionType('eq')
            ->setValue($scope)
            ->create();
        $this->searchCriteriaBuilder->addFilters([$filter]);
        $searchCriteria = $this->searchCriteriaBuilder->create();
        $tokenItems  = $this->tokenRepository->getList($searchCriteria)->getItems();

        if(sizeof($tokenItems) > 0){
            foreach ($tokenItems as $token) {
                return $token->getAccessToken();
            }
        }
    }

    public function setAccessToken($accessToken, $scope)
    {
        $filter = $this->filterBuilder
            ->setField(Token::SCOPE)
            ->setConditionType('eq')
            ->setValue($scope)
            ->create();
        $this->searchCriteriaBuilder->addFilters([$filter]);
        $searchCriteria = $this->searchCriteriaBuilder->create();
        $tokenItems  = $this->tokenRepository->getList($searchCriteria)->getItems();

        if(sizeof($tokenItems) > 0){
            foreach($tokenItems as $token){
                $token->setAccessToken($accessToken);
                $token->setUpdatedAt(time());
                try{
                    $token = $this->tokenRepository->save($token);
                }
                catch (Exception $e){
                    throw new CouldNotSaveException(__($e->getMessage()));
                }
            }
        }
        else{
            $token = $this->tokenFactory->create();
            $token->setAccessToken($accessToken);
            $token->setScope($scope);
            $token->setUpdatedAt(time());
            try{
                $token = $this->tokenRepository->save($token);
            }
            catch (Exception $e){
                throw new CouldNotSaveException(__($e->getMessage()));
            }
        }
    }
}
